/**
  * \file mrf49xa.c
  * \brief Utilisation du chip MRF49XA
  * \author Nicolas Husse
  * \version 0.1
  * \date 05 novembre 2013
  */

#include "mrf49xa.h"
#include <util/delay.h>
#include "serial.h"
#include <avr/wdt.h>

/**
 *  \fn mrf_reset
 *  \brief Reset du chip mrf
 */
void mrf_reset(void)
{
	mrf_port_reset &= ~(1 << mrf_bit_reset); 
	_delay_ms(10); 
	mrf_port_reset |= (1 << mrf_bit_reset);
}

/**
  * \fn mrf_init
  * \brief Initialisation du chip MRF49XA.
  */
void mrf_init(void)
{
	cli();
	
	// Initialisation du port d'interruption
	mrf_port_irq &= ~(1 << mrf_bit_irq);
	mrf_ddr_irq  &= ~(1 << mrf_bit_irq);
	
	EICRA = (1 << ISC11); // Utilisation de l'interruption
	
	// Initialisation du port Chip Select
	mrf_port_select  |=  (1 << mrf_bit_select);
	mrf_ddr_select   |=  (1 << mrf_bit_select);
	
	mrf_ddr_reset  |=  1 << mrf_bit_reset; // Reset en sortie
	
	mrf_reset();
	
	mrf_register_set(MRF_FIFORSTREG_SET); // Set FIFORSTREG.
	mrf_register_set(MRF_FIFORSTREG_SET | MRF_FSCF); // Enable synchronous latch from FIFORSTREG.
	mrf_register_set(MRF_GENCREG_SET); // Program frequency band and crystal load capacitance from GENCREG.
	mrf_register_set(MRF_AFCCREG_SET); // Enable AFC function from AFCCREG.
	mrf_register_set(MRF_CFSREG_SET); // Set center frequency through CFSREG for transmit or receive frequency.
	mrf_register_set(MRF_DCSREG_SET); // Set data rate through DRSREG.
	mrf_register_set(MRF_PMCREG | MRF_CLKODIS); // Enable required functions (transmit, receive, etc.) from PMCREG.
	mrf_register_set(MRF_RXCREG_SET); // Configure RXCREG.
	mrf_register_set(MRF_TXCREG_SET); // Configure TXCREG.
	mrf_register_set(MRF_BBFCREG | MRF_ACRLC | (4 & MRF_DQTI_MASK));
	mrf_register_set(MRF_PMCREG | MRF_CLKODIS | MRF_TXCEN); // Read STSREG.
	_delay_ms(5); // Wait for 5 ms for the oscillator to get stabilized.
	mrf_register_set(MRF_PMCREG | MRF_CLKODIS | MRF_RXCEN); // Enable FIFO for data reception.
	mrf_register_set(MRF_GENCREG_SET | MRF_FIFOEN);
	mrf_register_set(MRF_FIFORSTREG_SET); // Set FIFORSTREG.
	mrf_register_set(MRF_FIFORSTREG_SET | MRF_FSCF); // Enable synchronous latch from FIFORSTREG.
	
	int status = mrf_status_read(); // On lit le registre de statut afin de le rinitialiser.
	serial_send(status);
	
	mrf_status = mrf_idle_state; // Le mrf est actuellement au repos.
	mrf_package_present = false; // aucun paquet n'est disponible
	
	EIMSK = (1 << mrf_int_irq); // Activation de l'interruption.
	
	watchdog_init(); // Initialisation du watchdog (pour l'envoie qui bug...)
	
	sei();
}

/**
  * \fn mrf_package_send
  * \brief Envoie d'un paquet.
  * \param[in] paquet : Paquet  envoyer au MRF49XA
  * \return Retourne vrai si le paquet a bien t envoy, sinon faux.
  */
bool mrf_package_send(struct mrf_package * paquet)
{
	serial_send(0x22);
	
	unsigned char inc = 1;
	bool cont = true;
	
	do // On attends que le mrf soit de nouveau disponible
	{
		serial_send(mrf_status);
		cli();
		if(mrf_status == mrf_idle_state)
		{
			mrf_status = mrf_send_state;
			cont = false;
		}
		else
		{
			_delay_ms(1);
		}
		sei();
	} 
	while (inc < 10 && cont);
	
	if(inc >= 10) // Aprs 10 tentatives, on considre qu'il y a un problme et on reset le mrf
	{
		serial_send(0x23);
		
		mrf_init();
		
		return false;
	}
	
	mrf_send_buffer.package[0] = 0xAA; // Rveil du registre
	mrf_send_buffer.package[1] = 0xAA; // "
	mrf_send_buffer.package[2] = 0x2D; // Octet de synchronisation 1
	mrf_send_buffer.package[3] = 0xD4; // Octet de synchronisation 2
	mrf_send_buffer.package[4] = paquet->size; // Taille du paquet  envoyer
	for(unsigned char inc = 0; inc < paquet->size; inc++)
	{
		mrf_send_buffer.package[inc+5] = paquet->package[inc];
	}
	mrf_counter = 0;
	
	// Mis  jour des registres pour prparer le registre TX
	mrf_register_set(MRF_PMCREG);
	mrf_register_set(MRF_GENCREG_SET|MRF_TXDEN);
	mrf_register_set(MRF_PMCREG|MRF_TXCEN);
	
	return true;
}

/**
  * \fn mrf_package_received
  * \brief Rcupration d'un paquet reu.
  * \return: Retourne le paquet reu.
  */
volatile struct mrf_package * mrf_package_received(void)
{
	if(mrf_package_present) // Si un paquet est disponible, on retourne son pointeur...
	{
		mrf_package_present = false;
		return &mrf_receipt_buffer;
	}
	else // ...sinon on retourne 0
		return NULL;
}

/**
  * \fn mrf_status_read
  * \brief Lit le registre de statut du MRF49XA.
  * \return Retourne le mot reu
  */
unsigned int mrf_status_read(void)
{
	mrf_register_set(MRF_STSREG);
	return spi_received_word();
}

/** 
  * \fn mrf_data_received
  * \brief Lit un octet provenant du MRF49XA.
  * \return Retourne l'octet reu
  */
unsigned char mrf_data_received(void)
{
	mrf_register_set(MRF_RXFIFOREG);
	return spi_received();
}

/**
 *  \fn ISR
 *  \brief Interruption du watchdog
 */
ISR(WDT_vect)
{
	cli();
	serial_send(0x32);
	watchdog_stop(); // Arrt du watchdog
	
	mrf_init();
    mrf_status = mrf_idle_state;
	mrf_counter = 0;
	sei();
}

/** 
  * \fn ISR
  * \brief Interruption du MRF
  */
ISR(mrf_vect_irq, ISR_BLOCK)
{
	cli();
	watchdog_stop(); // Arrt du watchdog
	serial_send(0x65);
	
	serial_send(mrf_status);
	
	_delay_us(1);
	
	switch(mrf_status)
	{
		// En attente
		case mrf_idle_state :
			serial_send(0x50);
			mrf_counter = 0; // initialisation du compteur  0
			// premier octet = taille du paquet (on ne prend pas en compte les octets de rveils et de synchronisation).
			mrf_receipt_buffer.size = mrf_data_received();
			mrf_status = mrf_receive_state; // On indique que l'on passe en mode de reception de donne.
			serial_send(mrf_receipt_buffer.size);
			break;
			
		// Envoie d'un octet du paquet
		case mrf_send_state :
			serial_send(0x60);
			
			serial_send(mrf_counter);
			
			// On envoie l'octet mrf_counter au registre TX du MRF.
			if(mrf_counter == 0)
			{
				mrf_register_set(MRF_TXBREG | (mrf_send_buffer.package[mrf_counter] & MRF_TX_MASK));
			}
			else
			{
				mrf_byte_send(mrf_send_buffer.package[mrf_counter] & MRF_TX_MASK);
			}
				
			mrf_counter++;
			
			// Si on est au dernier octet, on remet le mrf en mode de rception.
			if(mrf_counter >= mrf_send_buffer.size + 5)
			{
				mrf_register_set(MRF_PMCREG | MRF_RXCEN); // On dsactive les interruptions de transmission.
				mrf_register_set(MRF_GENCREG_SET | MRF_FIFOEN); // On ractive la reception.
				mrf_register_set(MRF_FIFORSTREG_SET); 
				mrf_register_set(MRF_FIFORSTREG_SET | MRF_FSCF);
				
				mrf_status = mrf_idle_state; // Le mrf repasse au repos.
				mrf_counter = 0; // Le compteur est rinitialis.
			}
			serial_send(0x61);
			break;
			
		// Reception d'un octet
		case mrf_receive_state :
			serial_send(0x70);
			serial_send(mrf_receipt_buffer.size);
			mrf_receipt_buffer.package[mrf_counter++] = mrf_data_received();
			if(mrf_counter >= mrf_receipt_buffer.size)
			{
				mrf_package_present = true; // Un paquet est disponible
				mrf_status = mrf_idle_state; // Le mrf repasse au repos.
				mrf_register_set(MRF_FIFORSTREG_SET);
				mrf_register_set(MRF_FIFORSTREG_SET | MRF_FSCF);
			}
			serial_send(mrf_receipt_buffer.package[mrf_counter-1]);
			break;
			
		default :
			break;
	}
			
	serial_send(0x55);
	watchdog_start(); // Dmarrage du watchdog  4 secondes
	sei();
}