/**
  * \file mrf49xa.h
  * \brief Utilisation du chip MRF49XA
  * \author Nicolas Husse
  * \version 0.1
  * \date 07 novembre 2013
  */

#ifndef MRF49XA_H_
#define MRF49XA_H_

#include "config.h"
#include "spi.h"
#include "mrf49xa_registre.h"

/**
 * Ports utiliss par le MRF sur l'ATMega328p.
 */

#define mrf_port_select PORTB
#define mrf_pin_select	PINB
#define mrf_ddr_select	DDRB
#define mrf_bit_select	2

#define mrf_port_irq	PORTD
#define mrf_pin_irq		PIND
#define mrf_ddr_irq		DDRD
#define mrf_bit_irq		2
#define mrf_vect_irq	INT0_vect
#define mrf_int_irq		INT0

#define mrf_port_reset	PORTD
#define mrf_pin_reset	PIND
#define mrf_ddr_reset	DDRD
#define mrf_bit_reset	7

/** 
  * Valeurs des registres
  */
#define MRF_FBS			MRF_FBS_868
#define MRF_FREQB		868

#if(MRF_FBS == MRF_FBS_433)
#define MRF_FVAL		MRF_FVAL_433(MRF_FREQB)
#elif(MRF_FBS == MRF_FBS_868)
#define MRF_FVAL		MRF_FVAL_868(MRF_FREQB)
#elif(MRF_FBS == MRF_FBS_915)
#define MRF_FVAL		MRF_FVAL_915(MRF_FREQB)
#else
#error "Frquence non support."
#endif

#define MRF_LCS			MRF_LCS_160
#define MRF_ARFO		MRF_ARFO_0304
#define MRF_RXBW		MRF_RXBW_067
#define MRF_CBTC		MRF_CBTC_5
#define MRF_DRSSIT		MRF_DRSSIT_103
#define MRF_MODBW		MRF_MODBW_030
#define MRF_OTXPWR		MRF_OTXPWR_000
/** 
  * DRPV = 10000 / [29 x (1 + DRPE x 7) x DREx]  1 
  * DREx = dbit voulut = 100kbps
  * DRPV = 2
  */
#define MRF_DRPV		2
#define MRF_FFBC		8 /*!< Nombre de bit reu avant de lancer l'interruption. */

/** 
  * Configs des registres. Voir la datasheet et mrf49xa_registre pour plus de dtails.
  */
#define MRF_GENCREG_SET	(MRF_GENCREG | MRF_LCS | MRF_FBS) /**< Config du registre GENCREG. */
#define MRF_AFCCREG_SET	(MRF_AFCCREG | MRF_AUTOMS_INDP | MRF_ARFO | MRF_HAM | MRF_FOREN | MRF_FOFEN) /**< Config du registre AFCCEG. */
#define MRF_PLLCREG_SET (MRF_PLLCREG | MRF_CBTC) /**< Config du registre PLLCREG. */
#define MRF_CFSREG_SET	(MRF_CFSREG | (MRF_FREQB & MRF_FREQB_MASK)) /**< Coonfig du registre CFSREG. */
#define MRF_RXCREG_SET	(MRF_RXCREG | MRF_FINTDIO | MRF_RXBW | MRF_DRSSIT) /**< Coonfig du registre RXCREG. */
#define MRF_TXCREG_SET	(MRF_TXCREG | MRF_MODBW | MRF_OTXPWR) /**< Coonfig du registre TXCREG. */
#define MRF_DCSREG_SET	(MRF_DCSREG | (MRF_DRPV & MRF_DRPV_MASK)) /**< Coonfig du registre DCSREG. */
#define MRF_FIFORSTREG_SET	(MRF_FIFORSTREG | MRF_DRSTM | ((MRF_FFBC << 4) & MRF_FFBC_MASK)) /**< Coonfig du registre FIFORSTREG. */

/**
 *  \def mrf_select
 *  \brief Selectionner le MRF
 */
#define mrf_select() spi_chip_select(mrf_port_select, (unsigned char) mrf_bit_select);
/**
 *  \def mrf_unselect
 *  \brief Dselectionner le MRF
 */
#define mrf_unselect() spi_chip_unselect(mrf_port_select, (unsigned char) mrf_bit_select);
/**
 *  \def mrf_register_set
 *  \brief Envoyer un mot de 16 bits au MRF
 */
#define mrf_register_set(setting) mrf_select(); spi_send_word(setting); mrf_unselect();
/**
 *  \def mrf_byte_send
 *  \brief Envoyer un octet au MRF
 */
#define mrf_byte_send(byte) mrf_select(); spi_send(byte); mrf_unselect();

/**
 *  \def watchdog_start
 *  \brief Dmarrer le watchdog
 */
#define watchdog_start() WDTCSR = (1<<WDIE)|(1<<WDE)|(1<<WDP3);
/**
 *  \def watchdog_stop
 *  \brief Arrrer le watchdog
 */
#define watchdog_stop() wdt_disable();
/**
 *  \def watchdog_init
 *  \brief Initialiser le watchdog
 */
#define watchdog_init() wdt_reset(); WDTCSR = (1<<WDCE)|(1<<WDE);

#define mrf_package_size 8
/** 
  * \struct Paquet
  * \brief Paquet provenant ou  destination du MRF.
  */
struct mrf_package 
{
	unsigned char package[mrf_package_size+4];
	unsigned char size;
};

#define mrf_idle_state		0x00
#define mrf_send_state		0x01
#define mrf_receive_state	0x02

volatile unsigned char mrf_status;

volatile struct mrf_package mrf_receipt_buffer;
volatile struct mrf_package mrf_send_buffer;

unsigned char mrf_counter;
bool mrf_package_present;

/**
 *  \fn mrf_reset
 *  \brief Reset du chip mrf
 */
void mrf_reset(void);

/**
  * \fn mrf_init
  * \brief Initialisation du chip MRF49XA.
  */
void mrf_init(void);

/**
  * \fn mrf_package_send
  * \brief Envoie d'un paquet.
  * \param[in] paquet : Paquet  envoyer au MRF49XA
  * \return Retourne vrai si le paquet a bien t envoy, sinon faux.
  */
bool mrf_package_send(struct mrf_package * paquet);

/**
  * \fn mrf_package_received
  * \brief Reception d'un paquet.
  * \return: Retourne le paquet reu.
  */
volatile struct mrf_package * mrf_package_received(void);

/**
  * \fn mrf_status_read
  * \brief Lit le statut du MRF49XA.
  * \return Retourne le mot reu
  */
unsigned int mrf_status_read(void);

/** 
  * \fn mrf_data_received
  * \brief Lit un octet provenant du MRF49XA.
  * \return Retourne l'octet reu
  */
unsigned char mrf_data_received(void);

#endif /* MRF49XA_H_ */