/**
  * \file spi.c
  * \brief Communication par le port spi de l'ATMega
  * \author Nicolas Husse
  * \version 0.1
  * \date 05 novembre 2013
  */

#include "spi.h"

/**
  * \fn spi_init
  * \brief Initialisation de la communication spi.
  */
void spi_init(void)
{
	SPCR   =  0x50;	// Mode SPI
	SPSR   =  0x01;	// Double speed operation 

	SPI_MISO_PORT &= ~(1 << SPI_MISO_BIT); // On met le port  0
	SPI_MISO_DDR  &= ~(1 << SPI_MISO_BIT); // MISO en entre
	SPI_MOSI_PORT &= ~(1 << SPI_MOSI_BIT); // On met le port  0
	SPI_MOSI_DDR  |=  (1 << SPI_MOSI_BIT); // MOSI en sortie
	SPI_SCLK_PORT &= ~(1 << SPI_SCLK_BIT); // On met le port  0
	SPI_SCLK_DDR  |=  (1 << SPI_SCLK_BIT); // SCLK en sortie
	
	spi_buffer = 0;
	spi_mutex = false;
}

/**
 * \fn spi_chip_select
 * \param[in] port_select : Port  selectionner
 * \param[in] pin_select : Pin  selectionner
 */
void spi_chip_select(unsigned int port_select, char pin_select)
{
	port_select &= ~(1 << pin_select);
}

/**
 * \fn spi_chip_unselect
 * \param[in] port_select : Port  deselectionner
 * \param[in] pin_select : Pin  deselectionner
 */
void spi_chip_unselect(unsigned int port_select, char pin_select)
{
	port_select |= (1 << pin_select);
}

/**
  * \fn spi_send
  * \brief Envoie d'un octet sur le port spi.
  * \param[in] byteToSend : Octet  envoyer sur le port spi
  * \return Retourne la valeur reu sur le port spi
  */
unsigned char spi_send(unsigned char byteToSend)
{
	while (spi_mutex);
	spi_mutex = true;
	
	SPDR = byteToSend;
	
	do { } while ( (!(_SFR_BYTE(SPSR) & _BV(SPIF))) );
	
	spi_buffer = (unsigned int) SPDR;
	
	spi_mutex = false;
	
	return (unsigned char) (spi_buffer & 0x00FF);
}

/**
  * \fn spi_send_word
  * \brief Envoie d'un mot sur le port spi.
  * \param[in] wordToSend : Mot  envoyer sur le port spi
  * \return Retourne la valeur reu sur le port spi
  */
unsigned int spi_send_word(unsigned int wordToSend)
{
	while (spi_mutex);
	spi_mutex = true;
	
	SPDR = (unsigned char) ( (wordToSend & 0xFF00) >> 8 );
	
	do { } while ( (!(_SFR_BYTE(SPSR) & _BV(SPIF))) );
	
	spi_buffer = ((unsigned int) SPDR << 8) & 0xFF00;
	
	SPDR = (unsigned char) (wordToSend & 0x00FF);
	
	do { } while ( (!(_SFR_BYTE(SPSR) & _BV(SPIF))) );
	
	spi_buffer |= (unsigned int) SPDR & 0x00FF;
	
	spi_mutex = false;
	
	return spi_buffer;
}

/** 
  * \fn spi_received
  * \brief Lit un octet provenant du port spi.
  * \return Retourne l'octet reu
  */
unsigned char spi_received(void)
{
	while (spi_mutex);
	spi_mutex = true;
	
	unsigned char temp = (unsigned char) (spi_buffer & 0x00FF);
	
	spi_mutex = false;
	
	return temp;
}

/** 
  * \fn spi_received_word
  * \brief Lit un mot provenant du port spi.
  * \return Retourne le mot reu
  */
unsigned int spi_received_word(void)
{
	while (spi_mutex);
	spi_mutex = true;
	
	unsigned int temp = spi_buffer;
	
	spi_mutex = false;
	
	return temp;
}
